{*******************************************************}
{                                                       }
{         Delphi VCL eXtensions   (X)                   }
{                                                       }
{         Copyright (c) 1997 Alexander Buzaev           }
{                                                       }
{*******************************************************}

unit XButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RXCtrls, Menus;

const
  scAltDown = scAlt + vk_Down;
type
  TXSpeedButton = class(TRxSpeedButton)
  private
    FFocusControl: TWinControl;
    FClickKey: TShortCut;
    FNewWndProc: Pointer;
    FPrevWndProc: Pointer;
    procedure SetFocusControl(AControl: TWinControl);
  protected
    procedure HookWndProc(var AMsg: TMessage);
    procedure UnhookControl;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property FocusControl: TWinControl read FFocusControl write SetFocusControl;
    property ClickKey: TShortCut read FClickKey write FClickKey
      default scAltDown;
  end;

procedure Register;

implementation
type
  THack = class(TWinControl);


procedure Register;
begin
  RegisterComponents('X', [TXSpeedButton]);
end;

constructor TXSpeedButton.Create(AOwner: TComponent);
Begin
  inherited;
  FClickKey := scAltDown;
  FFocusControl:=Nil;
  FNewWndProc := MakeObjectInstance(HookWndProc);
  FPrevWndProc := nil;
End;

destructor TXSpeedButton.Destroy;
Begin
  FreeObjectInstance(FNewWndProc);
  FNewWndProc := nil;
  inherited;
End;

procedure TXSpeedButton.SetFocusControl(AControl: TWinControl);
var
  P: Pointer;
Begin
  IF AControl=Nil Then
  begin
    UnhookControl;
    FFocusControl:=Nil;
  end
  Else
  begin
    FFocusControl:=AControl;
    if Assigned(FFocusControl) and not ((csDesigning in FFocusControl.ComponentState) or
      (csDestroying in FFocusControl.ComponentState)) then
    begin
      FFocusControl.HandleNeeded;
      P := Pointer(GetWindowLong(FFocusControl.Handle, GWL_WNDPROC));
      if (P <> FNewWndProc) then begin
        FPrevWndProc := P;
        SetWindowLong(FFocusControl.Handle, GWL_WNDPROC, LongInt(FNewWndProc));
      end;
    end;
  end;
End;

procedure TXSpeedButton.HookWndProc(var AMsg: TMessage);
var
  ShiftState: TShiftState;
begin
  if Assigned(FFocusControl) then
  begin
    if (AMsg.Msg <> WM_QUIT) then
    begin//My Code
      if (AMsg.Msg=WM_KeyDown) or (AMsg.Msg=WM_SYSKEYDOWN) then
      begin
        ShiftState := KeyDataToShiftState(AMsg.lParam);
        if (ShortCut(AMsg.wParam, ShiftState)=FClickKey) Then
          OnClick(Self);
      end;
    end;
    with AMsg do
    begin
      if Assigned(FPrevWndProc) then
        Result := CallWindowProc(FPrevWndProc, FFocusControl.Handle, Msg,
          WParam, LParam)
      else
        Result := CallWindowProc(THack(FFocusControl).DefWndProc,
          FFocusControl.Handle, Msg, WParam, LParam);
      if Msg = WM_DESTROY then
        UnhookControl;
    end;//With
  end;
end;

procedure TXSpeedButton.UnhookControl;
begin
  if Assigned(FFocusControl) then begin
    if Assigned(FPrevWndProc) and FFocusControl.HandleAllocated and
    (Pointer(GetWindowLong(FFocusControl.Handle, GWL_WNDPROC)) = FNewWndProc) then
      SetWindowLong(FFocusControl.Handle, GWL_WNDPROC, LongInt(FPrevWndProc));
  end;
  FPrevWndProc := nil;
end;


end.
